# Stubs for rdflib.parser (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from io import StringIO as BytesIO

class Parser:
    def __init__(self): ...
    def parse(self, source, sink): ...



# Stubs for galaxy.tools.deps.containers (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from .container_resolvers.explicit import ExplicitContainerResolver as ExplicitContainerResolver
from .container_resolvers.mulled import BuildMulledDockerContainerResolver as BuildMulledDockerContainerResolver, BuildMulledSingularityContainerResolver as BuildMulledSingularityContainerResolver, CachedMulledDockerContainerResolver as CachedMulledDockerContainerResolver, CachedMulledSingularityContainerResolver as CachedMulledSingularityContainerResolver, MulledDockerContainerResolver as MulledDockerContainerResolver
from .requirements import ContainerDescription as ContainerDescription
from .requirements import DEFAULT_CONTAINER_RESOLVE_DEPENDENCIES as DEFAULT_CONTAINER_RESOLVE_DEPENDENCIES, DEFAULT_CONTAINER_SHELL as DEFAULT_CONTAINER_SHELL
from ..deps import docker_util as docker_util
from ..deps import singularity_util as singularity_util

log = ...  # type: Any
DOCKER_CONTAINER_TYPE = ...  # type: str
SINGULARITY_CONTAINER_TYPE = ...  # type: str
DEFAULT_CONTAINER_TYPE = ...  # type: Any
ALL_CONTAINER_TYPES = ...  # type: Any
LOAD_CACHED_IMAGE_COMMAND_TEMPLATE = ...  # type: str

class ContainerFinder:
    app_info = ...  # type: Any
    container_registry = ...  # type: Any
    def __init__(self, app_info) -> None: ...
    def find_best_container_description(self, enabled_container_types, tool_info): ...
    def find_container(self, tool_info, destination_info, job_info): ...

class NullContainerFinder:
    def find_container(self, tool_info, destination_info, job_info): ...

class ContainerRegistry:
    resolver_classes = ...  # type: Any
    enable_beta_mulled_containers = ...  # type: Any
    app_info = ...  # type: Any
    container_resolvers = ...  # type: Any
    def __init__(self, app_info) -> None: ...
    def find_best_container_description(self, enabled_container_types, tool_info): ...

class AppInfo:
    galaxy_root_dir = ...  # type: Any
    default_file_path = ...  # type: Any
    outputs_to_working_directory = ...  # type: Any
    container_image_cache_path = ...  # type: Any
    library_import_dir = ...  # type: Any
    enable_beta_mulled_containers = ...  # type: Any
    containers_resolvers_config_file = ...  # type: Any
    involucro_path = ...  # type: Any
    involucro_auto_init = ...  # type: Any
    def __init__(self, galaxy_root_dir: Optional[Any] = ..., default_file_path: Optional[Any] = ..., outputs_to_working_directory: bool = ..., container_image_cache_path: Optional[Any] = ..., library_import_dir: Optional[Any] = ..., enable_beta_mulled_containers: bool = ..., containers_resolvers_config_file: Optional[Any] = ..., involucro_path: Optional[Any] = ..., involucro_auto_init: bool = ...) -> None: ...

class ToolInfo:
    container_descriptions = ...  # type: Any
    requirements = ...  # type: Any
    requires_galaxy_python_environment = ...  # type: Any
    env_pass_through = ...  # type: Any
    def __init__(self, container_descriptions: Any = ..., requirements: Any = ..., requires_galaxy_python_environment: bool = ...) -> None: ...

class JobInfo:
    working_directory = ...  # type: Any
    job_directory = ...  # type: Any
    tool_directory = ...  # type: Any
    job_directory_type = ...  # type: Any
    def __init__(self, working_directory, tool_directory, job_directory, job_directory_type) -> None: ...

class Container:
    container_id = ...  # type: Any
    app_info = ...  # type: Any
    tool_info = ...  # type: Any
    destination_info = ...  # type: Any
    job_info = ...  # type: Any
    container_description = ...  # type: Any
    def __init__(self, container_id, app_info, tool_info, destination_info, job_info, container_description) -> None: ...
    @property
    def resolve_dependencies(self): ...
    @property
    def shell(self): ...
    def containerize_command(self, command): ...

def preprocess_volumes(volumes_raw_str, container_type): ...

class HasDockerLikeVolumes: ...

class DockerContainer(Container, HasDockerLikeVolumes):
    container_type = ...  # type: Any
    def containerize_command(self, command): ...

def docker_cache_path(cache_directory, container_id): ...

class SingularityContainer(Container, HasDockerLikeVolumes):
    container_type = ...  # type: Any
    def containerize_command(self, command): ...

CONTAINER_CLASSES = ...  # type: Any

class NullContainer:
    def __init__(self) -> None: ...
    def __bool__(self): ...
    __nonzero__ = ...  # type: Any

NULL_CONTAINER = ...  # type: Any



# Stubs for rdflib.exceptions (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class Error(Exception):
    msg = ... # type: Any
    def __init__(self, msg=None): ...

class TypeCheckError(Error):
    type = ... # type: Any
    node = ... # type: Any
    def __init__(self, node): ...

class SubjectTypeError(TypeCheckError):
    msg = ... # type: Any
    def __init__(self, node): ...

class PredicateTypeError(TypeCheckError):
    msg = ... # type: Any
    def __init__(self, node): ...

class ObjectTypeError(TypeCheckError):
    msg = ... # type: Any
    def __init__(self, node): ...

class ContextTypeError(TypeCheckError):
    msg = ... # type: Any
    def __init__(self, node): ...

class ParserError(Error):
    msg = ... # type: Any
    def __init__(self, msg): ...

class UniquenessError(Error):
    def __init__(self, values): ...



from .util import get_data, get_windows_safe_factory, windows_needs_docker

@windows_needs_docker
def test_newline_in_entry():
    """
    test that files in InitialWorkingDirectory are created with a newline character
    """
    factory = get_windows_safe_factory()
    echo = factory.make(get_data("tests/wf/iwdr-entry.cwl"))
    assert echo(message="hello") == {"out": "CONFIGVAR=hello\n"}



# Stubs for pathlib2 (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, AnyStr, Type, TypeVar, Optional, Union
from collections import Sequence

_P = TypeVar('_P', bound='PurePath')

intern = ...  # type: Any
basestring = ...  # type: Any
supports_symlinks = ...  # type: bool
nt = ...  # type: Any

class _Flavour:
    join = ...  # type: Any
    def __init__(self) -> None: ...
    def parse_parts(self, parts): ...
    def join_parsed_parts(self, drv, root, parts, drv2, root2, parts2): ...

class _WindowsFlavour(_Flavour):
    sep = ...  # type: str
    altsep = ...  # type: str
    has_drv = ...  # type: bool
    pathmod = ...  # type: Any
    is_supported = ...  # type: Any
    drive_letters = ...  # type: Any
    ext_namespace_prefix = ...  # type: str
    reserved_names = ...  # type: Any
    def splitroot(self, part, sep: Any = ...): ...
    def casefold(self, s): ...
    def casefold_parts(self, parts): ...
    def resolve(self, path): ...
    def is_reserved(self, parts): ...
    def make_uri(self, path): ...
    def gethomedir(self, username): ...

class _PosixFlavour(_Flavour):
    sep = ...  # type: str
    altsep = ...  # type: str
    has_drv = ...  # type: bool
    pathmod = ...  # type: Any
    is_supported = ...  # type: Any
    def splitroot(self, part, sep: Any = ...): ...
    def casefold(self, s): ...
    def casefold_parts(self, parts): ...
    def resolve(self, path): ...
    def is_reserved(self, parts): ...
    def make_uri(self, path): ...
    def gethomedir(self, username): ...

class _Accessor: ...

class _NormalAccessor(_Accessor):
    stat = ...  # type: Any
    lstat = ...  # type: Any
    open = ...  # type: Any
    listdir = ...  # type: Any
    chmod = ...  # type: Any
    lchmod = ...  # type: Any
    #def lchmod(self, pathobj, mode): ...
    mkdir = ...  # type: Any
    unlink = ...  # type: Any
    rmdir = ...  # type: Any
    rename = ...  # type: Any
    replace = ...  # type: Any
    symlink = ...  # type: Any
    #def symlink(a, b, target_is_directory): ...
    #@staticmethod
    #def symlink(a, b, target_is_directory): ...
    utime = ...  # type: Any
    def readlink(self, path): ...

class _Selector:
    child_parts = ...  # type: Any
    successor = ...  # type: Any
    def __init__(self, child_parts) -> None: ...
    def select_from(self, parent_path): ...

class _TerminatingSelector: ...

class _PreciseSelector(_Selector):
    name = ...  # type: Any
    def __init__(self, name, child_parts) -> None: ...

class _WildcardSelector(_Selector):
    pat = ...  # type: Any
    def __init__(self, pat, child_parts) -> None: ...

class _RecursiveWildcardSelector(_Selector):
    def __init__(self, pat, child_parts) -> None: ...

class _PathParents(Sequence):
    def __init__(self, path) -> None: ...
    def __len__(self): ...
    def __getitem__(self, idx): ...

class PurePath:
    def __new__(cls, *args): ...
    def __reduce__(self): ...
    def as_posix(self): ...
    def __bytes__(self): ...
    def as_uri(self) -> str: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __hash__(self): ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...
    drive = ...  # type: Any
    root = ...  # type: Any
    @property
    def anchor(self): ...
    @property
    def name(self): ...
    @property
    def suffix(self): ...
    @property
    def suffixes(self): ...
    @property
    def stem(self): ...
    def with_name(self, name): ...
    def with_suffix(self, suffix): ...
    def relative_to(self, *other): ...
    @property
    def parts(self): ...
    def joinpath(self, *args): ...
    def __truediv__(self, key): ...
    def __rtruediv__(self, key): ...
    __div__ = ...  # type: Any
    __rdiv__ = ...  # type: Any
    @property
    def parent(self): ...
    @property
    def parents(self): ...
    def is_absolute(self): ...
    def is_reserved(self): ...
    def match(self, path_pattern): ...

class PurePosixPath(PurePath): ...
class PureWindowsPath(PurePath): ...

class Path(PurePath):
    def __new__(cls: Type[_P], *args: Union[AnyStr, PurePath],
	**kwargs: Any) -> _P: ...
    def __enter__(self): ...
    def __exit__(self, t, v, tb): ...
    @classmethod
    def cwd(cls): ...
    @classmethod
    def home(cls): ...
    def samefile(self, other_path): ...
    def iterdir(self): ...
    def glob(self, pattern): ...
    def rglob(self, pattern): ...
    def absolute(self): ...
    def resolve(self): ...
    def stat(self): ...
    def owner(self): ...
    def group(self): ...
    def open(self, mode: str = ..., buffering: int = ..., encoding: Optional[Any] = ..., errors: Optional[Any] = ..., newline: Optional[Any] = ...): ...
    def read_bytes(self): ...
    def read_text(self, encoding: Optional[Any] = ..., errors: Optional[Any] = ...): ...
    def write_bytes(self, data): ...
    def write_text(self, data, encoding: Optional[Any] = ..., errors: Optional[Any] = ...): ...
    def touch(self, mode: int = ..., exist_ok: bool = ...): ...
    def mkdir(self, mode: int = ..., parents: bool = ..., exist_ok: bool = ...): ...
    def chmod(self, mode): ...
    def lchmod(self, mode): ...
    def unlink(self): ...
    def rmdir(self): ...
    def lstat(self): ...
    def rename(self, target): ...
    def replace(self, target): ...
    def symlink_to(self, target, target_is_directory: bool = ...): ...
    def exists(self): ...
    def is_dir(self): ...
    def is_file(self): ...
    def is_symlink(self): ...
    def is_block_device(self): ...
    def is_char_device(self): ...
    def is_fifo(self): ...
    def is_socket(self): ...
    def expanduser(self): ...

class PosixPath(Path, PurePosixPath): ...
class WindowsPath(Path, PureWindowsPath): ...



#!/usr/bin/env python

# Toy program to search inverted index and print out each line the term
# appears.

import sys

mainfile = sys.argv[1]
indexfile = sys.argv[1] + ".idx1"
term = sys.argv[2]

main = open(mainfile)
index = open(indexfile)

st = term + ": "

for a in index:
    if a.startswith(st):
        n = [int(i) for i in a[len(st):].split(", ") if i]
        linenum = 0
        for l in main:
            linenum += 1
            if linenum in n:
                print linenum, l.rstrip()
        break



# Stubs for avro (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Names in __all__ with no definition:
#   datafile
#   io
#   ipc
#   protocol
#   schema



# Stubs for rdflib.plugin (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from rdflib.exceptions import Error
from rdflib.parser import Parser

class PluginException(Error): ...

class Plugin:
    name = ... # type: Any
    kind = ... # type: Any
    module_path = ... # type: Any
    class_name = ... # type: Any
    def __init__(self, name, kind, module_path, class_name): ...
    def getClass(self): ...

class PKGPlugin(Plugin):
    name = ... # type: Any
    kind = ... # type: Any
    ep = ... # type: Any
    def __init__(self, name, kind, ep): ...
    def getClass(self): ...

def register(name: str, kind: type, module_path: str, class_name: str) -> None: ...
def get(name, kind): ...
def plugins(name=None, kind=None): ...



# Stubs for cachecontrol.caches.file_cache (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Callable
from ..cache import BaseCache as BaseCache
from ..controller import CacheController as CacheController

class FileCache(BaseCache):
    directory = ...  # type: str
    forever = ...  # type: bool
    filemode = ...  # type: str
    dirmode = ...  # type: str
    lock_class = ...  # type: Callable
    def __init__(self, directory: str, forever=False, filemode=384, dirmode=448, use_dir_lock=None, lock_class=None) -> None: ...
    @staticmethod
    def encode(x): ...
    def get(self, key): ...
    def set(self, key, value): ...
    def delete(self, key): ...

def url_to_file_path(url, filecache): ...



import pytest

from cwltool.secrets import SecretStore

from .util import get_data

@pytest.fixture
def secrets():
    sec_store = SecretStore()
    job = {'foo': 'bar',
           'baz': 'quux'}

    sec_store.store(['foo'], job)
    return sec_store, job

def test_obscuring(secrets):
    storage, obscured = secrets
    assert obscured['foo'] != 'bar'
    assert obscured['baz'] == 'quux'
    assert storage.retrieve(obscured)['foo'] == 'bar'

obscured_factories_expected = [
    ((lambda x: 'hello %s' % x), 'hello bar'),
    ((lambda x: ['echo', 'hello %s' % x]), ['echo', 'hello bar']),
    ((lambda x: {'foo': x}), {'foo': 'bar'}),
]

@pytest.mark.parametrize('factory,expected', obscured_factories_expected)
def test_secrets(factory, expected, secrets):
    storage, obscured = secrets
    pattern = factory(obscured['foo'])
    assert pattern != expected

    assert storage.has_secret(pattern)
    assert storage.retrieve(pattern) == expected

    assert obscured['foo'] != 'bar'
    assert obscured['baz'] == 'quux'



# Stubs for rdflib.store (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from rdflib.events import Event
from io import StringIO as BytesIO

class StoreCreatedEvent(Event): ...
class TripleAddedEvent(Event): ...
class TripleRemovedEvent(Event): ...

class NodePickler:
    def __init__(self): ...
    def register(self, object, id): ...
    def loads(self, s): ...
    def dumps(self, obj, protocol=None, bin=None): ...

class Store:
    context_aware = ... # type: Any
    formula_aware = ... # type: Any
    transaction_aware = ... # type: Any
    graph_aware = ... # type: Any
    dispatcher = ... # type: Any
    def __init__(self, configuration=None, identifier=None): ...
    node_pickler = ... # type: Any
    def create(self, configuration): ...
    def open(self, configuration, create=False): ...
    def close(self, commit_pending_transaction=False): ...
    def destroy(self, configuration): ...
    def gc(self): ...
    def add(self, __tuple_arg_2, context, quoted=False): ...
    def addN(self, quads): ...
    def remove(self, __tuple_arg_2, context=None): ...
    def triples_choices(self, __tuple_arg_2, context=None): ...
    def triples(self, triple_pattern, context=None): ...
    def __len__(self, context=None): ...
    def contexts(self, triple=None): ...
    def query(self, query, initNs, initBindings, queryGraph, **kwargs): ...
    def update(self, update, initNs, initBindings, queryGraph, **kwargs): ...
    def bind(self, prefix, namespace): ...
    def prefix(self, namespace): ...
    def namespace(self, prefix): ...
    def namespaces(self): ...
    def commit(self): ...
    def rollback(self): ...
    def add_graph(self, graph): ...
    def remove_graph(self, graph): ...



import unittest
import schema_salad.ref_resolver
import schema_salad.main
import schema_salad.schema
import rdflib
import yaml

class TestSchemas(unittest.TestCase):
    def test_schemas(self):
        l = schema_salad.ref_resolver.Loader({})

        ra, _ = l.resolve_all({
            "$schemas": ["tests/EDAM.owl"],
            "$namespaces": {"edam": "http://edamontology.org/"},
            "edam:has_format": "edam:format_1915"
            }, "")

        self.assertEquals(ra, {
            "$schemas": ["tests/EDAM.owl"],
            "$namespaces": {"edam": "http://edamontology.org/"},
            'http://edamontology.org/has_format': 'http://edamontology.org/format_1915'
        })



    # def test_domain(self):
    #     l = schema_salad.ref_resolver.Loader({})

    #     l.idx["http://example.com/stuff"] = {
    #         "$schemas": ["tests/EDAM.owl"],
    #         "$namespaces": {"edam": "http://edamontology.org/"},
    #     }

    #     ra, _ = l.resolve_all({
    #         "$import": "http://example.com/stuff",
    #         "edam:has_format": "edam:format_1915"
    #         }, "")

    #     self.assertEquals(ra, {
    #         "$schemas": ["tests/EDAM.owl"],
    #         "$namespaces": {"edam": "http://edamontology.org/"},
    #         'http://edamontology.org/has_format': 'http://edamontology.org/format_1915'
    #     })

    def test_self_validate(self):
        schema_salad.main.main(args=["schema_salad/metaschema/metaschema.yml"])
        schema_salad.main.main(args=["schema_salad/metaschema/metaschema.yml",
                                     "schema_salad/metaschema/metaschema.yml"])

    def test_jsonld_ctx(self):
        ldr, _, _ = schema_salad.schema.load_schema({
            "$base": "Y",
            "name": "X",
            "$namespaces": {
                "foo": "http://example.com/foo#"
            },
            "$graph": [{
                "name": "ExampleType",
                "type": "enum",
                "symbols": ["asym", "bsym"]}]
        })

        ra, _ = ldr.resolve_all({"foo:bar": "asym"}, "X")

        self.assertEquals(ra, {
            'http://example.com/foo#bar': 'asym'
        })

    def test_examples(self):
        self.maxDiff = None
        for a in ["field_name", "ident_res", "link_res", "vocab_res"]:
            ldr, _, _ = schema_salad.schema.load_schema("schema_salad/metaschema/%s_schema.yml" % a)
            src = ldr.resolve_all(yaml.load(open("schema_salad/metaschema/%s_src.yml" % a)), "")[0]
            proc = yaml.load(open("schema_salad/metaschema/%s_proc.yml" % a))
            self.assertEquals(proc, src)


if __name__ == '__main__':
    unittest.main()



# Stubs for ruamel.yaml.parser (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from ruamel.yaml.error import MarkedYAMLError

class ParserError(MarkedYAMLError): ...



# Stubs for galaxy.tools.deps.dependencies (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class DependenciesDescription:
    requirements = ...  # type: Any
    installed_tool_dependencies = ...  # type: Any
    def __init__(self, requirements: Any = ..., installed_tool_dependencies: Any = ...) -> None: ...
    def to_dict(self): ...
    @staticmethod
    def from_dict(as_dict): ...



# Stubs for galaxy.util (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
import collections
from six.moves.urllib import parse as urlparse, request as urlrequest
from .inflection import English as English, Inflector as Inflector

grp = ...  # type: Any
docutils_core = ...  # type: Any
docutils_html4css1 = ...  # type: Any
inflector = ...  # type: Any
log = ...  # type: Any
CHUNK_SIZE = ...  # type: int
DATABASE_MAX_STRING_SIZE = ...  # type: int
DATABASE_MAX_STRING_SIZE_PRETTY = ...  # type: str
gzip_magic = ...  # type: str
bz2_magic = ...  # type: str
DEFAULT_ENCODING = ...  # type: Any
NULL_CHAR = ...  # type: str
BINARY_CHARS = ...  # type: Any
FILENAME_VALID_CHARS = ...  # type: str

def remove_protocol_from_url(url): ...
def is_binary(value, binary_chars: Optional[Any] = ...): ...
def is_uuid(value): ...
def directory_hash_id(id): ...
def get_charset_from_http_headers(headers, default: Optional[Any] = ...): ...
def synchronized(func): ...
def file_iter(fname, sep: Optional[Any] = ...): ...
def file_reader(fp, chunk_size: Any = ...): ...
def unique_id(KEY_SIZE: int = ...): ...
def parse_xml(fname): ...
def parse_xml_string(xml_string): ...
def xml_to_string(elem, pretty: bool = ...): ...
def xml_element_compare(elem1, elem2): ...
def xml_element_list_compare(elem_list1, elem_list2): ...
def xml_element_to_dict(elem): ...
def pretty_print_xml(elem, level: int = ...): ...
def get_file_size(value, default: Optional[Any] = ...): ...
def shrink_stream_by_size(value, size, join_by: str = ..., left_larger: bool = ..., beginning_on_size_error: bool = ..., end_on_size_error: bool = ...): ...
def shrink_string_by_size(value, size, join_by: str = ..., left_larger: bool = ..., beginning_on_size_error: bool = ..., end_on_size_error: bool = ...): ...
def pretty_print_time_interval(time: bool = ..., precise: bool = ...): ...
def pretty_print_json(json_data, is_json_string: bool = ...): ...

valid_chars = ...  # type: Any
mapped_chars = ...  # type: Any

def restore_text(text, character_map: Any = ...): ...
def sanitize_text(text, valid_characters: Any = ..., character_map: Any = ..., invalid_character: str = ...): ...
def sanitize_lists_to_string(values, valid_characters: Any = ..., character_map: Any = ..., invalid_character: str = ...): ...
def sanitize_param(value, valid_characters: Any = ..., character_map: Any = ..., invalid_character: str = ...): ...

valid_filename_chars = ...  # type: Any
invalid_filenames = ...  # type: Any

def sanitize_for_filename(text, default: Optional[Any] = ...): ...
def mask_password_from_url(url): ...
def ready_name_for_url(raw_name): ...
def which(file): ...
def safe_makedirs(path): ...
def in_directory(file, directory, local_path_module: Any = ...): ...
def merge_sorted_iterables(operator, *iterables): ...

class Params:
    NEVER_SANITIZE = ...  # type: Any
    def __init__(self, params, sanitize: bool = ...) -> None: ...
    def flatten(self): ...
    def __getattr__(self, name): ...
    def get(self, key, default): ...
    def __len__(self): ...
    def __iter__(self): ...
    def update(self, values): ...

def rst_to_html(s): ...
def xml_text(root, name: Optional[Any] = ...): ...

truthy = ...  # type: Any
falsy = ...  # type: Any

def asbool(obj): ...
def string_as_bool(string): ...
def string_as_bool_or_none(string): ...
def listify(item, do_strip: bool = ...): ...
def commaify(amount): ...
def roundify(amount, sfs: int = ...): ...
def unicodify(value, encoding: Any = ..., error: str = ..., default: Optional[Any] = ...): ...
def smart_str(s, encoding: Any = ..., strings_only: bool = ..., errors: str = ...): ...
def object_to_string(obj): ...
def string_to_object(s): ...

class ParamsWithSpecs(collections.defaultdict):
    specs = ...  # type: Any
    params = ...  # type: Any
    def __init__(self, specs: Optional[Any] = ..., params: Optional[Any] = ...) -> None: ...
    def __missing__(self, name): ...
    def __getattr__(self, name): ...

def compare_urls(url1, url2, compare_scheme: bool = ..., compare_hostname: bool = ..., compare_path: bool = ...): ...
def read_dbnames(filename): ...
def read_build_sites(filename, check_builds: bool = ...): ...
def relativize_symlinks(path, start: Optional[Any] = ..., followlinks: bool = ...): ...
def stringify_dictionary_keys(in_dict): ...
def recursively_stringify_dictionary_keys(d): ...
def mkstemp_ln(src, prefix: str = ...): ...
def umask_fix_perms(path, umask, unmasked_perms, gid: Optional[Any] = ...): ...
def docstring_trim(docstring): ...
def nice_size(size): ...
def size_to_bytes(size): ...
def send_mail(frm, to, subject, body, config, html: Optional[Any] = ...): ...
def force_symlink(source, link_name): ...
def move_merge(source, target): ...
def safe_str_cmp(a, b): ...

galaxy_root_path = ...  # type: Any

def galaxy_directory(): ...
def config_directories_from_setting(directories_setting, galaxy_root: Any = ...): ...
def parse_int(value, min_val: Optional[Any] = ..., max_val: Optional[Any] = ..., default: Optional[Any] = ..., allow_none: bool = ...): ...
def parse_non_hex_float(s): ...
def build_url(base_url, port: int = ..., scheme: str = ..., pathspec: Optional[Any] = ..., params: Optional[Any] = ..., doseq: bool = ...): ...
def url_get(base_url, password_mgr: Optional[Any] = ..., pathspec: Optional[Any] = ..., params: Optional[Any] = ...): ...
def download_to_file(url, dest_file_path, timeout: int = ..., chunk_size: Any = ...): ...
def safe_relpath(path): ...

class ExecutionTimer:
    begin = ...  # type: Any
    def __init__(self) -> None: ...
    @property
    def elapsed(self): ...



# Stubs for posixpath (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from genericpath import *
from typing import Any, AnyStr, List

curdir: str
pardir: str
extsep: str
sep: str
pathsep: str
defpath: str
altsep: Any
devnull: str

def normcase(s): ...
def isabs(s: AnyStr) -> bool: ...
def join(a: AnyStr, *p: AnyStr) -> AnyStr: ...
def split(p: AnyStr) -> List[AnyStr]: ...
def splitext(p): ...
def splitdrive(p): ...
def basename(p: AnyStr) -> AnyStr: ...
def dirname(p): ...
def islink(path): ...
def lexists(path): ...
def samefile(f1, f2): ...
def sameopenfile(fp1, fp2): ...
def samestat(s1, s2): ...
def ismount(path): ...
def walk(top, func, arg): ...
def expanduser(path): ...
def expandvars(path): ...
def normpath(path): ...
def abspath(path): ...
def realpath(filename): ...

supports_unicode_filenames: Any

def relpath(path, start: Any = ...): ...

# Names in __all__ with no definition:
#   commonprefix
#   exists
#   getatime
#   getctime
#   getmtime
#   getsize
#   isdir
#   isfile



# Stubs for ruamel.yaml.main (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from ruamel.yaml.loader import *

def load(stream, Loader: Any =..., version=None) -> Any: ...



# Stubs for rdflib (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from rdflib.term import URIRef as URIRef, BNode as BNode, Literal as Literal, Variable as Variable
from rdflib.namespace import Namespace as Namespace
from rdflib.graph import Dataset as Dataset, Graph as Graph, ConjunctiveGraph as ConjunctiveGraph
from rdflib.namespace import RDF as RDF, RDFS as RDFS, OWL as OWL, XSD as XSD
from rdflib import util as util



# Stubs for cachecontrol.wrapper (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Union
from .adapter import CacheControlAdapter as CacheControlAdapter
from .cache import DictCache, BaseCache
import requests

def CacheControl(sess: requests.sessions.Session,
                 cache: Union[DictCache, BaseCache] = None,
                 cache_etags: bool = True,
                 serializer=None,
                 heuristic=None) -> requests.sessions.Session: ...



# Stubs for subprocess32 3.5.0-rc1
# adapted from
# https://github.com/python/typeshed/blob/8e62a79970227aa331f8070e64151002b7dfddea/stdlib/3/subprocess.pyi
# From the subprocess32 README
# > Timeout support backported from Python 3.3 is included.
# > The run() API from Python 3.5 was backported
# > Otherwise features are frozen at the 3.2 level.

import sys
from typing import Sequence, Any, Mapping, Callable, Tuple, IO, Optional, Union, List, Type, Text
from types import TracebackType

# We prefer to annotate inputs to methods (eg subprocess.check_call) with these
# union types. However, outputs (eg check_call return) and class attributes
# (eg TimeoutError.cmd) we prefer to annotate with Any, so the caller does not
# have to use an assertion to confirm which type.
#
# For example:
#
# try:
#    x = subprocess.check_output(["ls", "-l"])
#    reveal_type(x)  # Any, but morally is _TXT
# except TimeoutError as e:
#    reveal_type(e.cmd)  # Any, but morally is _CMD
_FILE = Union[None, int, IO[Any]]
_TXT = Union[bytes, Text]
_PATH = Union[bytes, Text]
_CMD = Union[_TXT, Sequence[_PATH]]
_ENV = Union[Mapping[bytes, _TXT], Mapping[Text, _TXT]]

class CompletedProcess:
    # morally: _CMD
    args = ...  # type: Any
    returncode = ...  # type: int
    # morally: Optional[_TXT]
    stdout = ...  # type: Any
    stderr = ...  # type: Any
    def __init__(self, args: _CMD,
                 returncode: int,
                 stdout: Optional[_TXT] = ...,
                 stderr: Optional[_TXT] = ...) -> None: ...
    def check_returncode(self) -> None: ...

# Nearly same args as Popen.__init__ except for timeout, input, and check
def run(args: _CMD,
        timeout: Optional[float] = ...,
        input: Optional[_TXT] = ...,
        check: bool = ...,
        bufsize: int = ...,
        executable: _PATH = ...,
        stdin: _FILE = ...,
        stdout: _FILE = ...,
        stderr: _FILE = ...,
        preexec_fn: Callable[[], Any] = ...,
        close_fds: bool = ...,
        shell: bool = ...,
        cwd: Optional[_PATH] = ...,
        env: Optional[_ENV] = ...,
        universal_newlines: bool = ...,
        startupinfo: Any = ...,
        creationflags: int = ...,
        restore_signals: bool = ...,
        start_new_session: bool = ...,
        pass_fds: Any = ...) -> CompletedProcess: ...

def call(args: _CMD,
         bufsize: int = ...,
         executable: _PATH = ...,
         stdin: _FILE = ...,
         stdout: _FILE = ...,
         stderr: _FILE = ...,
         preexec_fn: Callable[[], Any] = ...,
         close_fds: bool = ...,
         shell: bool = ...,
         cwd: Optional[_PATH] = ...,
         env: Optional[_ENV] = ...,
         universal_newlines: bool = ...,
         startupinfo: Any = ...,
         creationflags: int = ...,
         restore_signals: bool = ...,
         start_new_session: bool = ...,
         pass_fds: Any = ...,
         timeout: float = ...) -> int: ...

def check_call(args: _CMD,
               bufsize: int = ...,
               executable: _PATH = ...,
               stdin: _FILE = ...,
               stdout: _FILE = ...,
               stderr: _FILE = ...,
               preexec_fn: Callable[[], Any] = ...,
               close_fds: bool = ...,
               shell: bool = ...,
               cwd: Optional[_PATH] = ...,
               env: Optional[_ENV] = ...,
               universal_newlines: bool = ...,
               startupinfo: Any = ...,
               creationflags: int = ...,
               restore_signals: bool = ...,
               start_new_session: bool = ...,
               pass_fds: Any = ...,
               timeout: float = ...) -> int: ...

def check_output(args: _CMD,
                 bufsize: int = ...,
                 executable: _PATH = ...,
                 stdin: _FILE = ...,
                 stderr: _FILE = ...,
                 preexec_fn: Callable[[], Any] = ...,
                 close_fds: bool = ...,
                 shell: bool = ...,
                 cwd: Optional[_PATH] = ...,
                 env: Optional[_ENV] = ...,
                 universal_newlines: bool = ...,
                 startupinfo: Any = ...,
                 creationflags: int = ...,
                 restore_signals: bool = ...,
                 start_new_session: bool = ...,
                 pass_fds: Any = ...,
                 timeout: float = ...,
                 ) -> Any: ...  # morally: -> _TXT


PIPE = ...  # type: int
STDOUT = ...  # type: int
DEVNULL = ...  # type: int
class SubprocessError(Exception): ...
class TimeoutExpired(SubprocessError):
# morally: _CMD
	cmd = ...  # type: Any
	timeout = ...  # type: float
	# morally: Optional[_TXT]
	output = ...  # type: Any
	stdout = ...  # type: Any
	stderr = ...  # type: Any


class CalledProcessError(Exception):
    returncode = 0
    # morally: _CMD
    cmd = ...  # type: Any
    # morally: Optional[_TXT]
    output = ...  # type: Any

    stdout = ...  # type: Any
    stderr = ...  # type: Any

    def __init__(self,
                 returncode: int,
                 cmd: _CMD,
                 output: Optional[_TXT] = ...,
                 stderr: Optional[_TXT] = ...) -> None: ...

class Popen:
    args = ...  # type: _CMD
    stdin = ...  # type: IO[Any]
    stdout = ...  # type: IO[Any]
    stderr = ...  # type: IO[Any]
    pid = 0
    returncode = 0


    def __init__(self,
                 args: _CMD,
                 bufsize: int = ...,
                 executable: Optional[_PATH] = ...,
                 stdin: Optional[_FILE] = ...,
                 stdout: Optional[_FILE] = ...,
                 stderr: Optional[_FILE] = ...,
                 preexec_fn: Optional[Callable[[], Any]] = ...,
                 close_fds: bool = ...,
                 shell: bool = ...,
                 cwd: Optional[_PATH] = ...,
                 env: Optional[_ENV] = ...,
                 universal_newlines: bool = ...,
                 startupinfo: Optional[Any] = ...,
                 creationflags: int = ...,
                 restore_signals: bool = ...,
                 start_new_session: bool = ...,
                 pass_fds: Any = ...) -> None: ...

    def poll(self) -> int: ...
    def wait(self, timeout: Optional[float] = ...) -> int: ...
    # Return str/bytes
    def communicate(self,
                    input: Optional[_TXT] = ...,
                    timeout: Optional[float] = ...,
                    # morally: -> Tuple[Optional[_TXT], Optional[_TXT]]
                    ) -> Tuple[Any, Any]: ...
    def send_signal(self, signal: int) -> None: ...
    def terminate(self) -> None: ...
    def kill(self) -> None: ...
    def __enter__(self) -> 'Popen': ...
    def __exit__(self, type: Optional[Type[BaseException]], value: Optional[BaseException], traceback: Optional[TracebackType]) -> bool: ...

# The result really is always a str.
def getstatusoutput(cmd: _TXT) -> Tuple[int, str]: ...
def getoutput(cmd: _TXT) -> str: ...

def list2cmdline(seq: Sequence[str]) -> str: ...  # undocumented



# Stubs for avro.schema (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Union, Dict, List, Text

PRIMITIVE_TYPES = ... # type: Any
NAMED_TYPES = ... # type: Any
VALID_TYPES = ... # type: Any
SCHEMA_RESERVED_PROPS = ... # type: Any
FIELD_RESERVED_PROPS = ... # type: Any
VALID_FIELD_SORT_ORDERS = ... # type: Any

class AvroException(Exception): ...
class SchemaParseException(AvroException): ...

class Schema:
    type = ... # type: Any
    def __init__(self, type, other_props=None): ...
    props = ... # type: Any
    other_props = ... # type: Any
    def get_prop(self, key): ...
    def set_prop(self, key, value): ...
    def to_json(self, names): ...

class Name:
    def __init__(self, name_attr, space_attr, default_space): ...
    def __eq__(self, other): ...
    fullname = ... # type: Any
    def get_space(self): ...

class Names:
    names = ... # type: Any
    default_namespace = ... # type: Any
    def __init__(self, default_namespace: Union[str, Text] = None) -> None: ...
    def has_name(self, name_attr: Union[str, Text], space_attr: Union[str, Text]) -> bool: ...
    def get_name(self, name_attr: Union[str, Text], space_attr: Union[str, Text]) -> Schema: ...
    def prune_namespace(self, properties): ...
    def add_name(self, name_attr, space_attr, new_schema): ...

class NamedSchema(Schema):
    def __init__(self, type, name, namespace=None, names=None, other_props=None): ...
    def name_ref(self, names): ...
    name = ... # type: Any
    namespace = ... # type: Any
    fullname = ... # type: Any

class Field:
    type = ... # type: Any
    name = ... # type: Any
    def __init__(self, type, name, has_default, default=None, order=None, names=None, doc=None, other_props=None): ...
    default = ... # type: Any
    has_default = ... # type: Any
    order = ... # type: Any
    doc = ... # type: Any
    props = ... # type: Any
    other_props = ... # type: Any
    def get_prop(self, key): ...
    def set_prop(self, key, value): ...
    def to_json(self, names=None): ...
    def __eq__(self, that): ...

class PrimitiveSchema(Schema):
    fullname = ... # type: Any
    def __init__(self, type, other_props=None): ...
    def to_json(self, names=None): ...
    def __eq__(self, that): ...

class FixedSchema(NamedSchema):
    def __init__(self, name, namespace, size, names=None, other_props=None): ...
    size = ... # type: Any
    def to_json(self, names=None): ...
    def __eq__(self, that): ...

class EnumSchema(NamedSchema):
    def __init__(self, name, namespace, symbols, names=None, doc=None, other_props=None): ...
    symbols = ... # type: Any
    doc = ... # type: Any
    def to_json(self, names=None): ...
    def __eq__(self, that): ...

class ArraySchema(Schema):
    def __init__(self, items, names=None, other_props=None): ...
    items = ... # type: Any
    def to_json(self, names=None): ...
    def __eq__(self, that): ...

class MapSchema(Schema):
    def __init__(self, values, names=None, other_props=None): ...
    values = ... # type: Any
    def to_json(self, names=None): ...
    def __eq__(self, that): ...

class UnionSchema(Schema):
    def __init__(self, schemas, names=None): ...
    schemas = ... # type: Any
    def to_json(self, names=None): ...
    def __eq__(self, that): ...

class ErrorUnionSchema(UnionSchema):
    def __init__(self, schemas, names=None): ...
    def to_json(self, names=None): ...

class RecordSchema(NamedSchema):
    @staticmethod
    def make_field_objects(field_data, names): ...
    def __init__(self, name, namespace, fields, names=None, schema_type='', doc=None, other_props=None): ...
    fields = ... # type: Any
    doc = ... # type: Any
    @property
    def fields_dict(self): ...
    def to_json(self, names=None): ...
    def __eq__(self, that): ...

def get_other_props(all_props, reserved_props): ...
def make_avsc_object(json_data: Union[Dict, List, str], names: Names) -> Schema: ...
def parse(json_string): ...



# Stubs for rdflib.term (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Text

class Node: ...

class Identifier(Node, Text):
    def __new__(cls, value): ...
    def eq(self, other): ...
    def neq(self, other): ...
    def __ne__(self, other): ...
    def __eq__(self, other): ...
    def __gt__(self, other): ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __ge__(self, other): ...
    def __hash__(self): ...

class URIRef(Identifier):
    def __new__(cls, value, base=None): ...
    def toPython(self): ...
    def n3(self, namespace_manager=None): ...
    def defrag(self): ...
    def __reduce__(self): ...
    def __getnewargs__(self): ...
    def __add__(self, other): ...
    def __radd__(self, other): ...
    def __mod__(self, other): ...
    def md5_term_hash(self): ...
    def de_skolemize(self): ...

class Genid(URIRef): ...
class RDFLibGenid(Genid): ...

class BNode(Identifier):
    def __new__(cls, value=None, _sn_gen=..., _prefix=...): ...
    def toPython(self): ...
    def n3(self, namespace_manager=None): ...
    def __getnewargs__(self): ...
    def __reduce__(self): ...
    def md5_term_hash(self): ...
    def skolemize(self, authority=''): ...

class Literal(Identifier):
    __doc__ = ... # type: Any
    def __new__(cls, lexical_or_value, lang=None, datatype=None, normalize=None): ...
    def normalize(self): ...
    @property
    def value(self): ...
    @property
    def language(self): ...
    @property
    def datatype(self): ...
    def __reduce__(self): ...
    def __add__(self, val): ...
    def __nonzero__(self): ...
    def __neg__(self): ...
    def __pos__(self): ...
    def __abs__(self): ...
    def __invert__(self): ...
    def __gt__(self, other): ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __ge__(self, other): ...
    def __hash__(self): ...
    def __eq__(self, other): ...
    def eq(self, other): ...
    def neq(self, other): ...
    def n3(self, namespace_manager=None): ...
    def toPython(self): ...
    def md5_term_hash(self): ...

def bind(datatype, pythontype, constructor=None, lexicalizer=None): ...

class Variable(Identifier):
    def __new__(cls, value): ...
    def toPython(self): ...
    def n3(self, namespace_manager=None): ...
    def __reduce__(self): ...
    def md5_term_hash(self): ...

class Statement(Node, tuple):
    def __new__(cls, __tuple_arg_2, context): ...
    def __reduce__(self): ...
    def toPython(self): ...



# Stubs for rdflib.query (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from io import StringIO as BytesIO
import collections

class Processor:
    def __init__(self, graph): ...
    def query(self, strOrQuery, initBindings=..., initNs=..., DEBUG=False): ...

class UpdateProcessor:
    def __init__(self, graph): ...
    def update(self, strOrQuery, initBindings=..., initNs=...): ...

class ResultException(Exception): ...

class EncodeOnlyUnicode:
    def __init__(self, stream): ...
    def write(self, arg): ...
    def __getattr__(self, name): ...

class ResultRow(tuple):
    __doc__ = ... # type: Any
    def __new__(cls, values, labels): ...
    def __getattr__(self, name): ...
    def __getitem__(self, name): ...
    def asdict(self): ...

class Result(collections.Iterable):
    type = ... # type: Any
    vars = ... # type: Any
    askAnswer = ... # type: Any
    graph = ... # type: Any
    def __init__(self, type_): ...
    bindings = ... # type: Any
    @staticmethod
    def parse(source, format='', **kwargs): ...
    def serialize(self, destination=None, encoding='', format='', **args): ...
    def __len__(self): ...
    def __nonzero__(self): ...
    def __iter__(self): ...
    def __getattr__(self, name): ...
    def __eq__(self, other): ...

class ResultParser:
    def __init__(self): ...
    def parse(self, source, **kwargs): ...

class ResultSerializer:
    result = ... # type: Any
    def __init__(self, result): ...
    def serialize(self, stream, encoding='', **kwargs): ...



# Stubs for rdflib.serializer (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class Serializer:
    store = ... # type: Any
    encoding = ... # type: Any
    base = ... # type: Any
    def __init__(self, store): ...
    def serialize(self, stream, base=None, encoding=None, **args): ...
    def relativize(self, uri): ...



from collections import namedtuple

svmem = namedtuple(
    'svmem', ['total', 'available', 'percent', 'used', 'free',
              'active', 'inactive', 'buffers', 'cached', 'shared', 'slab'])

def virtual_memory() -> svmem: ...

def cpu_count() -> int: ...


# Stubs for rdflib.graph (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, AnyStr, Dict, Union, IO, Tuple, Iterator, Text
from io import StringIO as BytesIO
from rdflib.term import Node, URIRef
from rdflib.store import Store
from rdflib.namespace import NamespaceManager
from rdflib.query import Result

class Graph(Node):
    context_aware = ... # type: Any
    formula_aware = ... # type: Any
    default_union = ... # type: Any
    def __init__(self, store: Union[Store, str]='default', identifier: Node=None, namespace_manager: NamespaceManager=None) -> None: ... 
    store = ... # type: Any
    identifier = ... # type: Any
    namespace_manager = ... # type: Any
    def toPython(self): ...
    def destroy(self, configuration): ...
    def commit(self): ...
    def rollback(self): ...
    def open(self, configuration, create=False): ...
    def close(self, commit_pending_transaction=False): ...
    def add(self, __tuple_arg_2: Tuple[Node, Node, Node]) -> None: ...
    def addN(self, quads): ...
    def remove(self, __tuple_arg_2: Tuple[Union[AnyStr, URIRef], AnyStr, Union[AnyStr, URIRef]]) -> None: ...
    def triples(self, __tuple_arg_2: Tuple[Union[AnyStr, URIRef, None], Union[AnyStr, None], Union[AnyStr, URIRef, None]]) -> Iterator[Tuple[AnyStr, AnyStr, AnyStr]]: ...
    def __getitem__(self, item): ...
    def __len__(self): ...
    def __iter__(self): ...
    def __contains__(self, triple): ...
    def __hash__(self): ...
    def md5_term_hash(self): ...
    def __cmp__(self, other): ...
    def __eq__(self, other): ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...
    def __iadd__(self, other): ...
    def __isub__(self, other): ...
    def __add__(self, other): ...
    def __mul__(self, other): ...
    def __sub__(self, other): ...
    def __xor__(self, other): ...
    __or__ = ... # type: Any
    __and__ = ... # type: Any
    def set(self, triple): ...
    def subjects(self, predicate=None, object=None): ...
    def predicates(self, subject=None, object=None): ...
    def objects(self, subject=None, predicate=None): ...
    def subject_predicates(self, object=None): ...
    def subject_objects(self, predicate=None): ...
    def predicate_objects(self, subject=None): ...
    def triples_choices(self, __tuple_arg_2, context=None): ...
    def value(self, subject=None, predicate=..., object=None, default=None, any=True): ...
    def label(self, subject, default=''): ...
    def preferredLabel(self, subject, lang=None, default=None, labelProperties=...): ...
    def comment(self, subject, default=''): ...
    def items(self, list): ...
    def transitiveClosure(self, func, arg, seen=None): ...
    def transitive_objects(self, subject, property, remember=None): ...
    def transitive_subjects(self, predicate, object, remember=None): ...
    def seq(self, subject): ...
    def qname(self, uri): ...
    def compute_qname(self, uri, generate=True): ...
    def bind(self, prefix: str, namespace: str, override: bool=True) -> None: ...
    def namespaces(self): ...
    def absolutize(self, uri, defrag=1): ...
    def serialize(self, destination: Union[Text, IO[Any]]=None, format: Text='', base: str=None, encoding: str=None, **args) -> Union[bytes, None]: ...
    def parse(self, source: str = None, publicID: str = None,
              format: Text = None,
              location: Text = None, file: IO[Any] = None,
              data: str = None, **args): ...
    def load(self, source, publicID=None, format=''): ...
    def query(self, query_object, processor: str = '', result: str = '', initNs: Dict = None, initBindings: Dict = None, use_store_provided: bool = True, **kwargs) -> Result: ...
    def update(self, update_object, processor='', initNs=..., initBindings=..., use_store_provided=True, **kwargs): ...
    def n3(self): ...
    def __reduce__(self): ...
    def isomorphic(self, other): ...
    def connected(self): ...
    def all_nodes(self): ...
    def resource(self, identifier): ...
    def skolemize(self, new_graph=None, bnode=None): ...
    def de_skolemize(self, new_graph=None, uriref=None): ...

class ConjunctiveGraph(Graph):
    context_aware = ... # type: Any
    default_union = ... # type: Any
    default_context = ... # type: Any
    def __init__(self, store='', identifier=None): ...
    def __contains__(self, triple_or_quad): ...
    def add(self, triple_or_quad): ...
    def addN(self, quads): ...
    def remove(self, triple_or_quad): ...
    def triples(self, triple_or_quad, context=None): ...
    def quads(self, triple_or_quad=None): ...
    def triples_choices(self, __tuple_arg_2, context=None): ...
    def __len__(self): ...
    def contexts(self, triple=None): ...
    def get_context(self, identifier, quoted=False): ...
    def remove_context(self, context): ...
    def context_id(self, uri, context_id=None): ...
    def parse(self, source=None, publicID=None, format='', location=None, file=None, data=None, **args): ...
    def __reduce__(self): ...

class Dataset(ConjunctiveGraph):
    __doc__ = ... # type: Any
    default_context = ... # type: Any
    default_union = ... # type: Any
    def __init__(self, store='', default_union=False): ...
    def graph(self, identifier=None): ...
    def parse(self, source=None, publicID=None, format='', location=None, file=None, data=None, **args): ...
    def add_graph(self, g): ...
    def remove_graph(self, g): ...
    def contexts(self, triple=None): ...
    def quads(self, quad): ...

class QuotedGraph(Graph):
    def __init__(self, store, identifier): ...
    def add(self, __tuple_arg_2): ...
    def addN(self, quads): ...
    def n3(self): ...
    def __reduce__(self): ...

class Seq:
    def __init__(self, graph, subject): ...
    def toPython(self): ...
    def __iter__(self): ...
    def __len__(self): ...
    def __getitem__(self, index): ...

class ModificationException(Exception):
    def __init__(self): ...

class UnSupportedAggregateOperation(Exception):
    def __init__(self): ...

class ReadOnlyGraphAggregate(ConjunctiveGraph):
    graphs = ... # type: Any
    def __init__(self, graphs, store=''): ...
    def destroy(self, configuration): ...
    def commit(self): ...
    def rollback(self): ...
    def open(self, configuration, create=False): ...
    def close(self): ...
    def add(self, __tuple_arg_2): ...
    def addN(self, quads): ...
    def remove(self, __tuple_arg_2): ...
    def triples(self, __tuple_arg_2): ...
    def __contains__(self, triple_or_quad): ...
    def quads(self, __tuple_arg_2): ...
    def __len__(self): ...
    def __hash__(self): ...
    def __cmp__(self, other): ...
    def __iadd__(self, other): ...
    def __isub__(self, other): ...
    def triples_choices(self, __tuple_arg_2, context=None): ...
    def qname(self, uri): ...
    def compute_qname(self, uri, generate=True): ...
    def bind(self, prefix, namespace, override=True): ...
    def namespaces(self): ...
    def absolutize(self, uri, defrag=1): ...
    def parse(self, source, publicID=None, format='', **args): ...
    def n3(self): ...
    def __reduce__(self): ...



# Stubs for rdflib.namespace (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Tuple, Union, Text

class Namespace(Text):
    __doc__ = ... # type: Any
    def __new__(cls, value): ...
    @property
    def title(self): ...
    def term(self, name): ...
    def __getitem__(self, key, default=None): ...
    def __getattr__(self, name): ...

class URIPattern(Text):
    __doc__ = ... # type: Any
    def __new__(cls, value): ...
    def __mod__(self, *args, **kwargs): ...
    def format(self, *args, **kwargs): ...

class ClosedNamespace:
    uri = ... # type: Any
    def __init__(self, uri, terms): ...
    def term(self, name): ...
    def __getitem__(self, key, default=None): ...
    def __getattr__(self, name): ...

class _RDFNamespace(ClosedNamespace):
    def __init__(self): ...
    def term(self, name): ...

RDF = ... # type: Any
RDFS = ... # type: Any
OWL = ... # type: Any
XSD = ... # type: Any
SKOS = ... # type: Any
DOAP = ... # type: Any
FOAF = ... # type: Any
DC = ... # type: Any
DCTERMS = ... # type: Any
VOID = ... # type: Any

class NamespaceManager:
    graph = ... # type: Any
    def __init__(self, graph): ...
    def reset(self): ...
    store = ... # type: Any
    def qname(self, uri): ...
    def normalizeUri(self, rdfTerm): ...
    def compute_qname(self, uri, generate=True): ...
    def bind(self, prefix, namespace, override=True, replace=False): ...
    def namespaces(self): ...
    def absolutize(self, uri, defrag=1): ...

def is_ncname(name): ...

XMLNS = ... # type: Any

def split_uri(uri: Text) -> Tuple[str, str]: ...



# Stubs for rdflib.events (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

class Event:
    def __init__(self, **kw): ...

class Dispatcher:
    def set_map(self, amap): ...
    def get_map(self): ...
    def subscribe(self, event_type, handler): ...
    def dispatch(self, event): ...



# Stubs for ruamel.yaml.error (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class YAMLError(Exception): ...

class MarkedYAMLError(YAMLError): ...



# Stubs for rdflib.util (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def list2set(seq): ...
def first(seq): ...
def uniq(sequence, strip=0): ...
def more_than(sequence, number): ...
def to_term(s, default=None): ...
def from_n3(s, default=None, backend=None): ...
def check_context(c): ...
def check_subject(s): ...
def check_predicate(p): ...
def check_object(o): ...
def check_statement(triple): ...
def check_pattern(triple): ...
def date_time(t=None, local_time_zone=False): ...
def parse_date_time(val): ...
def guess_format(fpath, fmap=None): ...
def find_roots(graph, prop, roots=None): ...
def get_tree(graph, root, prop, mapper=..., sortkey=None, done=None, dir=''): ...



# Stubs for avro.schema (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

logger = ...  # type: Any
DEBUG_VERBOSE = ...  # type: int
NULL = ...  # type: str
BOOLEAN = ...  # type: str
STRING = ...  # type: str
BYTES = ...  # type: str
INT = ...  # type: str
LONG = ...  # type: str
FLOAT = ...  # type: str
DOUBLE = ...  # type: str
FIXED = ...  # type: str
ENUM = ...  # type: str
RECORD = ...  # type: str
ERROR = ...  # type: str
ARRAY = ...  # type: str
MAP = ...  # type: str
UNION = ...  # type: str
REQUEST = ...  # type: str
ERROR_UNION = ...  # type: str
PRIMITIVE_TYPES = ...  # type: Any
NAMED_TYPES = ...  # type: Any
VALID_TYPES = ...  # type: Any
SCHEMA_RESERVED_PROPS = ...  # type: Any
FIELD_RESERVED_PROPS = ...  # type: Any
VALID_FIELD_SORT_ORDERS = ...  # type: Any

class Error(Exception): ...
class AvroException(Error): ...
class SchemaParseException(AvroException): ...

class ImmutableDict(dict):
    def __init__(self, items: Optional[Any] = ..., **kwargs) -> None: ...
    def __setitem__(self, key, value): ...
    def __delitem__(self, key): ...
    def clear(self): ...
    def update(self, **kwargs): ...
    def pop(self, key, default: Optional[Any] = ...): ...
    def popitem(self): ...

class Schema:
    def __init__(self, type, other_props: Optional[Any] = ...) -> None: ...
    @property
    def name(self): ...
    @property
    def fullname(self): ...
    @property
    def namespace(self): ...
    @property
    def type(self): ...
    @property
    def doc(self): ...
    @property
    def props(self): ...
    @property
    def other_props(self): ...
    def to_json(self, names): ...

class Name:
    def __init__(self, name, namespace: Optional[Any] = ...) -> None: ...
    def __eq__(self, other): ...
    @property
    def simple_name(self): ...
    @property
    def namespace(self): ...
    @property
    def fullname(self): ...

class Names:
    def __init__(self, default_namespace: Optional[Any] = ..., names: Optional[Any] = ...) -> None: ...
    @property
    def names(self): ...
    @property
    def default_namespace(self): ...
    def NewWithDefaultNamespace(self, namespace): ...
    def GetName(self, name, namespace: Optional[Any] = ...): ...
    def has_name(self, name, namespace: Optional[Any] = ...): ...
    def get_name(self, name, namespace: Optional[Any] = ...): ...
    def GetSchema(self, name, namespace: Optional[Any] = ...): ...
    def prune_namespace(self, properties): ...
    def Register(self, schema): ...

class NamedSchema(Schema):
    def __init__(self, type, name, namespace: Optional[Any] = ..., names: Optional[Any] = ..., other_props: Optional[Any] = ...) -> None: ...
    @property
    def avro_name(self): ...
    @property
    def name(self): ...
    @property
    def namespace(self): ...
    @property
    def fullname(self): ...
    def name_ref(self, names): ...

class Field:
    def __init__(self, type, name, index, has_default, default: Any = ..., order: Optional[Any] = ..., names: Optional[Any] = ..., doc: Optional[Any] = ..., other_props: Optional[Any] = ...) -> None: ...
    @property
    def type(self): ...
    @property
    def name(self): ...
    @property
    def index(self): ...
    @property
    def default(self): ...
    @property
    def has_default(self): ...
    @property
    def order(self): ...
    @property
    def doc(self): ...
    @property
    def props(self): ...
    @property
    def other_props(self): ...
    def to_json(self, names: Optional[Any] = ...): ...
    def __eq__(self, that): ...

class PrimitiveSchema(Schema):
    def __init__(self, type, other_props: Optional[Any] = ...) -> None: ...
    @property
    def name(self): ...
    def to_json(self, names: Optional[Any] = ...): ...
    def __eq__(self, that): ...

class FixedSchema(NamedSchema):
    def __init__(self, name, namespace, size, names: Optional[Any] = ..., other_props: Optional[Any] = ...) -> None: ...
    @property
    def size(self): ...
    def to_json(self, names: Optional[Any] = ...): ...
    def __eq__(self, that): ...

class EnumSchema(NamedSchema):
    def __init__(self, name, namespace, symbols, names: Optional[Any] = ..., doc: Optional[Any] = ..., other_props: Optional[Any] = ...) -> None: ...
    @property
    def symbols(self): ...
    def to_json(self, names: Optional[Any] = ...): ...
    def __eq__(self, that): ...

class ArraySchema(Schema):
    def __init__(self, items, other_props: Optional[Any] = ...) -> None: ...
    @property
    def items(self): ...
    def to_json(self, names: Optional[Any] = ...): ...
    def __eq__(self, that): ...

class MapSchema(Schema):
    def __init__(self, values, other_props: Optional[Any] = ...) -> None: ...
    @property
    def values(self): ...
    def to_json(self, names: Optional[Any] = ...): ...
    def __eq__(self, that): ...

class UnionSchema(Schema):
    def __init__(self, schemas) -> None: ...
    @property
    def schemas(self): ...
    def to_json(self, names: Optional[Any] = ...): ...
    def __eq__(self, that): ...

class ErrorUnionSchema(UnionSchema):
    def __init__(self, schemas) -> None: ...
    def to_json(self, names: Optional[Any] = ...): ...

class RecordSchema(NamedSchema):
    def __init__(self, name, namespace, fields: Optional[Any] = ..., make_fields: Optional[Any] = ..., names: Optional[Any] = ..., record_type: Any = ..., doc: Optional[Any] = ..., other_props: Optional[Any] = ...) -> None: ...
    @property
    def fields(self): ...
    @property
    def field_map(self): ...
    def to_json(self, names: Optional[Any] = ...): ...
    def __eq__(self, that): ...

def FilterKeysOut(items, keys): ...
def SchemaFromJSONData(json_data, names: Optional[Any] = ...): ...
def Parse(json_string): ...



# Stubs for pkg_resources (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Text
from os import open as os_open
from collections import namedtuple

require = ... # type: Any
working_set = ... # type: Any

class PEP440Warning(RuntimeWarning): ...

class _SetuptoolsVersionMixin:
    def __hash__(self): ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __eq__(self, other): ...
    def __ge__(self, other): ...
    def __gt__(self, other): ...
    def __ne__(self, other): ...
    def __getitem__(self, key): ...
    def __iter__(self): ...

def parse_version(v): ...

class ResolutionError(Exception): ...

class VersionConflict(ResolutionError):
    @property
    def dist(self): ...
    @property
    def req(self): ...
    def report(self): ...
    def with_context(self, required_by): ...

class ContextualVersionConflict(VersionConflict):
    @property
    def required_by(self): ...

class DistributionNotFound(ResolutionError):
    @property
    def req(self): ...
    @property
    def requirers(self): ...
    @property
    def requirers_str(self): ...
    def report(self): ...

class UnknownExtra(ResolutionError): ...

EGG_DIST = ... # type: Any
BINARY_DIST = ... # type: Any
SOURCE_DIST = ... # type: Any
CHECKOUT_DIST = ... # type: Any
DEVELOP_DIST = ... # type: Any

def register_loader_type(loader_type, provider_factory): ...
def get_provider(moduleOrReq): ...

get_platform = ... # type: Any

def compatible_platforms(provided, required): ...
def run_script(dist_spec, script_name): ...

run_main = ... # type: Any

def get_distribution(dist): ...
def load_entry_point(dist, group, name): ...
def get_entry_map(dist, group=None): ...
def get_entry_info(dist, group, name): ...

class IMetadataProvider:
    def has_metadata(name): ...
    def get_metadata(name): ...
    def get_metadata_lines(name): ...
    def metadata_isdir(name): ...
    def metadata_listdir(name): ...
    def run_script(script_name, namespace): ...

class IResourceProvider(IMetadataProvider):
    def get_resource_filename(manager, resource_name): ...
    def get_resource_stream(manager, resource_name): ...
    def get_resource_string(manager, resource_name): ...
    def has_resource(resource_name): ...
    def resource_isdir(resource_name): ...
    def resource_listdir(resource_name): ...

class WorkingSet:
    entries = ... # type: Any
    entry_keys = ... # type: Any
    by_key = ... # type: Any
    callbacks = ... # type: Any
    def __init__(self, entries=None): ...
    def add_entry(self, entry): ...
    def __contains__(self, dist): ...
    def find(self, req): ...
    def iter_entry_points(self, group, name=None): ...
    def run_script(self, requires, script_name): ...
    def __iter__(self): ...
    def add(self, dist, entry=None, insert=True, replace=False): ...
    def resolve(self, requirements, env=None, installer=None, replace_conflicting=False): ...
    def find_plugins(self, plugin_env, full_env=None, installer=None, fallback=True): ...
    def require(self, *requirements): ...
    def subscribe(self, callback): ...

class _ReqExtras(dict):
    def markers_pass(self, req): ...

class Environment:
    platform = ... # type: Any
    python = ... # type: Any
    def __init__(self, search_path=None, platform=..., python=...): ...
    def can_add(self, dist): ...
    def remove(self, dist): ...
    def scan(self, search_path=None): ...
    def __getitem__(self, project_name): ...
    def add(self, dist): ...
    def best_match(self, req, working_set, installer=None): ...
    def obtain(self, requirement, installer=None): ...
    def __iter__(self): ...
    def __iadd__(self, other): ...
    def __add__(self, other): ...

AvailableDistributions = ... # type: Any

class ExtractionError(RuntimeError): ...

class ResourceManager:
    extraction_path = ... # type: Any
    cached_files = ... # type: Any
    def __init__(self): ...
    def resource_exists(self, package_or_requirement, resource_name): ...
    def resource_isdir(self, package_or_requirement, resource_name): ...
    def resource_filename(self, package_or_requirement, resource_name): ...
    def resource_stream(self, package_or_requirement, resource_name): ...
    def resource_string(self, package_or_requirement, resource_name): ...
    def resource_listdir(self, package_or_requirement, resource_name): ...
    def extraction_error(self): ...
    def get_cache_path(self, archive_name, names=...): ...
    def postprocess(self, tempname, filename): ...
    def set_extraction_path(self, path): ...
    def cleanup_resources(self, force=False): ...

def get_default_cache(): ...
def safe_name(name): ...
def safe_version(version): ...
def safe_extra(extra): ...
def to_filename(name): ...
def invalid_marker(text): ...
def evaluate_marker(text, extra=None): ...

class NullProvider:
    egg_name = ... # type: Any
    egg_info = ... # type: Any
    loader = ... # type: Any
    module_path = ... # type: Any
    def __init__(self, module): ...
    def get_resource_filename(self, manager, resource_name): ...
    def get_resource_stream(self, manager, resource_name): ...
    def get_resource_string(self, manager, resource_name): ...
    def has_resource(self, resource_name): ...
    def has_metadata(self, name): ...
    def get_metadata(self, name): ...
    def get_metadata_lines(self, name): ...
    def resource_isdir(self, resource_name): ...
    def metadata_isdir(self, name): ...
    def resource_listdir(self, resource_name): ...
    def metadata_listdir(self, name): ...
    def run_script(self, script_name, namespace): ...

class EggProvider(NullProvider):
    def __init__(self, module): ...

class DefaultProvider(EggProvider):
    def get_resource_stream(self, manager, resource_name): ...

class EmptyProvider(NullProvider):
    module_path = ... # type: Any
    def __init__(self): ...

empty_provider = ... # type: Any

class ZipManifests(dict):
    @classmethod
    def build(cls, path): ...
    load = ... # type: Any

class ZipProvider(EggProvider):
    eagers = ... # type: Any
    zip_pre = ... # type: Any
    def __init__(self, module): ...
    @property
    def zipinfo(self): ...
    def get_resource_filename(self, manager, resource_name): ...

class FileMetadata(EmptyProvider):
    path = ... # type: Any
    def __init__(self, path): ...
    def has_metadata(self, name): ...
    def get_metadata(self, name): ...
    def get_metadata_lines(self, name): ...

class PathMetadata(DefaultProvider):
    module_path = ... # type: Any
    egg_info = ... # type: Any
    def __init__(self, path, egg_info): ...

class EggMetadata(ZipProvider):
    zip_pre = ... # type: Any
    loader = ... # type: Any
    module_path = ... # type: Any
    def __init__(self, importer): ...

def register_finder(importer_type, distribution_finder): ...
def find_distributions(path_item, only=False): ...
def register_namespace_handler(importer_type, namespace_handler): ...
def declare_namespace(packageName): ...
def fixup_namespace_packages(path_item, parent=None): ...
def normalize_path(filename): ...
def yield_lines(strs): ...

class EntryPoint:
    name = ... # type: Any
    module_name = ... # type: Any
    attrs = ... # type: Any
    extras = ... # type: Any
    dist = ... # type: Any
    def __init__(self, name, module_name, attrs=..., extras=..., dist=None): ...
    def load(self, require=True, *args, **kwargs): ...
    def resolve(self): ...
    def require(self, env=None, installer=None): ...
    pattern = ... # type: Any
    @classmethod
    def parse(cls, src, dist=None): ...
    @classmethod
    def parse_group(cls, group, lines, dist=None): ...
    @classmethod
    def parse_map(cls, data, dist=None): ...

class Distribution:
    PKG_INFO = ... # type: Any
    project_name = ... # type: Any
    py_version = ... # type: Any
    platform = ... # type: Any
    location = ... # type: Any
    precedence = ... # type: Any
    def __init__(self, location=None, metadata=None, project_name=None, version=None, py_version=..., platform=None, precedence=...): ...
    @classmethod
    def from_location(cls, location, basename, metadata=None, **kw): ...
    @property
    def hashcmp(self): ...
    def __hash__(self): ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    @property
    def key(self): ...
    @property
    def parsed_version(self): ...
    @property
    def version(self): ...
    def requires(self, extras=...): ...
    def activate(self, path=None): ...
    def egg_name(self): ...
    def __getattr__(self, attr): ...
    @classmethod
    def from_filename(cls, filename, metadata=None, **kw): ...
    def as_requirement(self): ...
    def load_entry_point(self, group, name): ...
    def get_entry_map(self, group=None): ...
    def get_entry_info(self, group, name): ...
    def insert_on(self, path, loc=None, replace=False): ...
    def check_version_conflict(self): ...
    def has_version(self): ...
    def clone(self, **kw): ...
    @property
    def extras(self): ...

class EggInfoDistribution(Distribution): ...

class DistInfoDistribution(Distribution):
    PKG_INFO = ... # type: Any
    EQEQ = ... # type: Any

class RequirementParseError(ValueError): ...

def parse_requirements(strs): ...

def ensure_directory(path): ...
def split_sections(s): ...

def resource_stream(package_or_requirement: Text, resource_name: Text): ...

# Names in __all__ with no definition:
#   add_activation_listener
#   cleanup_resources
#   iter_entry_points
#   resource_exists
#   resource_filename
#   resource_isdir
#   resource_listdir
#   resource_stream
#   resource_string
#   set_extraction_path



"""Test the ref_resolver module."""

from __future__ import absolute_import
import shutil
import tempfile

import pytest  # type: ignore

@pytest.fixture
def tmp_dir_fixture(request):
    d = tempfile.mkdtemp()

    @request.addfinalizer
    def teardown():
        shutil.rmtree(d)
    return d

def test_Loader_initialisation_for_HOME_env_var(tmp_dir_fixture):
    import os
    from schema_salad.ref_resolver import Loader
    from requests import Session

    # Ensure HOME is set.
    os.environ["HOME"] = tmp_dir_fixture

    loader = Loader(ctx={})
    assert isinstance(loader.session, Session)

def test_Loader_initialisation_for_TMP_env_var(tmp_dir_fixture):
    import os
    from schema_salad.ref_resolver import Loader
    from requests import Session

    # Ensure HOME is missing.
    if "HOME" in os.environ:
        del os.environ["HOME"]
    # Ensure TMP is present.
    os.environ["TMP"] = tmp_dir_fixture

    loader = Loader(ctx={})
    assert isinstance(loader.session, Session)

def test_Loader_initialisation_with_neither_TMP_HOME_set(tmp_dir_fixture):
    import os
    from schema_salad.ref_resolver import Loader
    from requests import Session

    # Ensure HOME is missing.
    if "HOME" in os.environ:
        del os.environ["HOME"]
    if "TMP" in os.environ:
        del os.environ["TMP"]

    loader = Loader(ctx={})
    assert isinstance(loader.session, Session)



#!/usr/bin/env python

# Toy program to generate inverted index of word to line.
# Takes input text file on stdin and prints output index on stdout.

import sys

words = {}

mainfile = sys.argv[1]
indexfile = sys.argv[1] + ".idx"

main = open(mainfile)
index = open(indexfile, "w")

linenum = 0
for l in main:
    linenum += 1
    l = l.rstrip().lower().replace(".", "").replace(",", "").replace(";", "").replace("-", " ")
    for w in l.split(" "):
        if w:
            if w not in words:
                words[w] = set()
            words[w].add(linenum)

for w in sorted(words.keys()):
    index.write("%s: %s" % (w, ", ".join((str(i) for i in words[w]))) + "\n")
